%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% FIRLS for total variation (TV) minimization
%%% minimize_y 0.5*||Ay-b|| + lambda*||D y||_1

% %%% Input:
% input.A: the sensing matrix
% input.b: the measurement
% input.D1, input.D2: discrete finite difference matrices for TV
% input.lambda: regularization parameter
% input.cgiter: number of inner CG iterations
% input.f: the groudtruth. Used for calculating RMSE etc.
% input.ratio: the mean value of AtA
% input.tol: stopping tolerance
% input.l: the lower bound of x, may be 0 or -inf
% input.u; the upper bound of x, may be 1, 255 or inf
% input.n1, input.n2: the size of the image

% %%% Output
% output.y: the reconstructed image
% output.rel: the relative error
% output.snr: the SNR
% output.xtime: CPU time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Contact
%%%% Chen Chen (chenchen.cn87@gmail.com)
%%%% Junzhou Huang (jzhuang@uta.edu) University of Texas at Arlington

%%%% Related Papers
%%%% Chen Chen, Junzhou Huang, Lei He, and Hongsheng Li. "Preconditioning for accelerated iteratively reweighted least squares in structured sparsity reconstruction."
%%%% In IEEE Conference on Computer Vision and Pattern Recognition (CVPR) , pp. 2713-2720. IEEE, 2014.

%%%% Chen, Chen, Junzhou Huang, Lei He, and Hongsheng Li. "Fast Iteratively Reweighted Least Squares Algorithms for Analysis-Based Sparsity Reconstruction." arXiv preprint arXiv:1411.5057 (2014).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function output = FIRLS_TV(input,varargin)

l=input.l; u=input.u;
if((l==-Inf)&&(u==Inf))
    project=@(x)x;
elseif (isfinite(l)&&(u==Inf))
    project=@(x)(((l<x).*x)+(l*(x<=l)));
elseif (isfinite(u)&&(l==-Inf))
    project=@(x)(((x<u).*x)+((x>=u)*u));
elseif ((isfinite(u)&&isfinite(l))&&(l<u))
    project=@(x)(((l<x)&(x<u)).*x)+((x>=u)*u)+(l*(x<=l));
else
    error('lower and upper bound l,u should satisfy l<u');
end

ratio =  input.ratio;
n1=input.n1; n2=input.n2; N=n1*n2;
A=input.A; b=input.b;
At = input.At;
Atb=At*b;
y= Atb;

lambda = input.lambda;

if isnumeric(A)
    AtA=At*A;
else
    AtA =  A_operator(@(x) At*(A*x), @(x) At*(A*x));
end
D1 = input.D1;
D2 = input.D2;
In = speye(N);
f0 = input.f;

t00 = cputime;
for itr = 1:input.no    % total iter counter
    
    
    yp=y;
    Q1= (abs(D1*y)+eps).^(2);
    Q2= (abs(D2*y)+eps).^(2);
    Q = (Q1+Q2).^(-0.5);
    
    P=ratio*In;
    tp=[(1:N)' (1:N)' Q];
    W=spconvert(tp); % weight
    tp =  lambda*((D1'*W*D1)+(D2'*W*D2));
    P= P+ tp; % preconditioner
    S = A_operator(@(x) AtA*x + tp*x, @(x) AtA*x + tp*x);
    
    [L,U] = inexactLUdecomposition(P, n1, N);
    
    invp = @(x) FFP(L,U,x);
    invP =  A_operator(@(x) invp(x), @(x) invp(x));
    
    y =  PCG_operator(S,Atb,invP,input.cgiter,y,1e-15,1,f0);
    
    y = project(y);
    
    output.rel(itr)=norm(y-yp, 'fro')/norm(yp, 'fro');
    output.snr(itr)=snr(y, input.f);
    output.xtime(itr)=cputime-t00;
    
    if(output.rel(end)<input.tol)
        display(['Done at iteration ' num2str(itr)]);
        break;
    end
    
end
output.y=y;

function [L,U] = inexactLUdecomposition(P, n1, N)

d = diag(P);
d1 = diag(P,1);
d2 = diag(P,n1);
d = diag(d);
d1 = diag(d1,1);
d2 = diag(d2,n1);
U = d +d1 +d2;
d1t = diag(P,-1);
d2t = diag(P,-n1);
ratios = 1./(diag(d));
d1t = ratios(1:N-1).*d1t;
d2t = ratios(1:N-n1).*d2t;
In = speye(N);
L = In+diag(d1t,-1)+diag(d2t,-n1);
